//-----------------------------------------------------------------------------
// DIRECTGRAPHICS.CPP - (c)2001 by Andr Knecke
//-----------------------------------------------------------------------------
// DirectX 8 Functions for DirectPac
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "directpac.h"


//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
LPDIRECT3D8             g_pD3D       = NULL;	// Used to create the D3DDevice
LPDIRECT3DDEVICE8       g_pd3dDevice = NULL;	// Our rendering device
LPD3DXSPRITE			g_lpSprite	 = NULL;	// Pointer to our Sprite-Device

LPDIRECT3DTEXTURE8 ppTexture[NUMTEXTURES];		// Space For Our Textures


//----------------------------------------------------------------------------
// LoadTexture() : Load a texture from a file, returns ptr to texture.
//                   D3DX8 supports BMP, PPM, DDS, JPG, PNG, TGA & DIB. 
//----------------------------------------------------------------------------
LPDIRECT3DTEXTURE8 LoadTexture(char *filename)
{
	LPDIRECT3DTEXTURE8 pd3dTexture;
    // Set black as our source color key. Use 0xFFFF00FF for magenta instead.
    // Use 0x00000000 for no 'color key' substitution
    D3DCOLOR colorkey = 0xFFFF00FF;
    // The D3DX function will fill in the following image info for us 
    D3DXIMAGE_INFO SrcInfo; // Optional 

    // Load image from file - maintain size of original file.
    // It is also possible to specify a new height/width and ask D3DX to filter
    // the image to the new size (stretch/shrink). See SDK docs for details.
    if (D3D_OK == (D3DXCreateTextureFromFileEx( g_pd3dDevice, filename, 0, 0, 1, 0, 
         D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, D3DX_FILTER_NONE , D3DX_DEFAULT, 
         colorkey, &SrcInfo , NULL, &pd3dTexture)))
		return pd3dTexture;
    else
		return NULL;
}


//----------------------------------------------------------------------------
// BltSprite() : A replacement of the DirectDraw Blt() function for Direct3D8
// Supports DDBLTFX_MIRRORLEFTRIGHT and DDBLTFX_MIRRORUPDOWN
// If pDestRect is NULL, copies it to the same co-ord as src
// If pSrcRect is NULL, copies the entire texture
//----------------------------------------------------------------------------
HRESULT BltSprite( RECT *pDestRect, LPDIRECT3DTEXTURE8 pSrcTexture, 
          RECT *pSrcRect, DWORD dwFlags )
{
#define DDBLTFX_MIRRORLEFTRIGHT 0
#define DDBLTFX_MIRRORUPDOWN 0

     D3DXVECTOR2 scaling(1,1), rcenter(0,0), trans(0,0);

     // Set the parameters - translation (screen location)
     if (pDestRect) {
          trans.x = (float) pDestRect->left;
          trans.y = (float) pDestRect->top;
     } 
     // Scaling - If Source & Destination are different size rects, then scale
     if (pDestRect && pSrcRect) {
          scaling.x = ((float) (pDestRect->right - pDestRect->left)) / 
               ((float) (pSrcRect->right - pSrcRect->left));
          scaling.y = ((float) (pDestRect->bottom - pDestRect->top)) / 
               ((float) (pSrcRect->bottom - pSrcRect->top));
     }
     // Handle the flags - need to change scaling & adjust translation
     if (pSrcRect && dwFlags) {
          if (dwFlags&DDBLTFX_MIRRORLEFTRIGHT) { 
               scaling.x = -scaling.x; 
               trans.x += (float) (pDestRect->right - pDestRect->left); 
          }
          if (dwFlags&DDBLTFX_MIRRORUPDOWN) { 
               scaling.y = -scaling.y; 
               trans.y += (float) (pDestRect->bottom - pDestRect->top); 
          }
     }

     // And here we go:
     return g_lpSprite->Draw( pSrcTexture, pSrcRect, &scaling, 
               &rcenter, 0, &trans, 0xFFFFFFFF );

}


//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Draws the scene
//-----------------------------------------------------------------------------
bool Render()
{
    if( NULL == g_pd3dDevice )
    {
		MessageBox( NULL, "Failed To Render Scene!\n --> Error Number 2", "CRITICAL ERROR", MB_ICONERROR);
		return false;
	}

	if (NULL == g_lpSprite )
	{
		MessageBox( NULL, "Failed To Render Scene!\n --> Error Number 3", "CRITICAL ERROR", MB_ICONERROR);
		return false;
	}

    // Clear the backbuffer to a blue color
    g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB(0,0,0), 1.0f, 0 );
    
    // Begin the scene
    g_pd3dDevice->BeginScene();
    
    // Rendering of scene objects can happen here
    g_lpSprite->Begin(); // Prepare for Drawing in 2D

	
	Draw();	// Draw The Stuff!
	

	g_lpSprite->End();

    // End the scene
    g_pd3dDevice->EndScene();
    
    // Present the backbuffer contents to the display
    g_pd3dDevice->Present( NULL, NULL, NULL, NULL );


	return true;
}


//-----------------------------------------------------------------------------
// Name: CreateDirectGraphicsWindow()
// Desc: Creates a Fullscreen-DirectGraphics 8 "Window"
//-----------------------------------------------------------------------------
bool CreateDirectGraphicsWindow(char* title, int width, int height, int bits, bool fullscreen)
{
	// Create the application's window
    hWnd = CreateWindow( "DirectGraphics", title, 
                              /*WS_OVERLAPPEDWINDOW|*/WS_SYSMENU|WS_MINIMIZEBOX, 100, 100, width, height,
                              GetDesktopWindow(), NULL, wc.hInstance, NULL );
	
    // Initialize Direct3D:
    // Create the D3D object, which is needed to create the D3DDevice.
    if( NULL == ( g_pD3D = Direct3DCreate8( D3D_SDK_VERSION ) ) )
        return false;

    // Get the current desktop display mode
    D3DDISPLAYMODE d3ddm;
    if( FAILED( g_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm ) ) )
        return false;

    // Set up the structure used to create the D3DDevice. Most parameters are
    // zeroed out. We set Windowed to TRUE, since we want to do D3D in a
    // window, and then set the SwapEffect to "discard", which is the most
    // efficient method of presenting the back buffer to the display.  And 
    // we request a back buffer format that matches the current desktop display 
    // format.
    D3DPRESENT_PARAMETERS d3dpp; 
    
	ZeroMemory( &d3dpp, sizeof(d3dpp) );
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferWidth = width;
    d3dpp.BackBufferHeight = height;
	if (!fullscreen)
		d3dpp.Windowed = true;
    d3dpp.BackBufferFormat = d3ddm.Format;

    // Create the Direct3D device. Here we are using the default adapter (most
    // systems only have one, unless they have multiple graphics hardware cards
    // installed) and requesting the HAL (which is saying we want the hardware
    // device rather than a software one). Software vertex processing is 
    // specified since we know it will work on all cards. On cards that support 
    // hardware vertex processing, though, we would see a big performance gain 
    // by specifying hardware vertex processing.
    if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                      &d3dpp, &g_pd3dDevice ) ) )
    {
        return false;
    }

	// Create The Sprite Interface:
	D3DXCreateSprite(g_pd3dDevice, &g_lpSprite);
	
	
	// Call The Init-Function
	Init(); // (in Main.cpp)
	

    // Show the window
    ShowWindow( hWnd, SW_SHOWDEFAULT );
    UpdateWindow( hWnd );

	return true;
}


//-----------------------------------------------------------------------------
// Name: KillDirectGraphicsWindow()
// Desc: Releases all previously initialized objects
//-----------------------------------------------------------------------------
void KillDirectGraphicsWindow()
{
	if (fs)
		ChangeDisplaySettings(NULL ,0);

	if( g_lpSprite != NULL)
		g_lpSprite->Release();
		
	if( g_pd3dDevice != NULL) 
        g_pd3dDevice->Release();
	
    if( g_pD3D != NULL)
        g_pD3D->Release();
	
	if (hWnd && !DestroyWindow(hWnd))					// Are We Able To Destroy The Window?
	{
		MessageBox(NULL,"Could Not Release hWnd!","SHUTDOWN ERROR",MB_ICONERROR);
		hWnd=NULL;										// Set hWnd To NULL
	}

}




